@@ -14,6 +14,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import docker
+import json
+import re
+
+from ansible.module_utils.basic import AnsibleModule
+
 DOCUMENTATION = '''
 ---
 module: kolla_toolbox
@@ -44,6 +50,12 @@ options:
     required: False
     type: str
     default: auto
+  timeout:
+    description:
+      - The default timeout for docker-py client when contacting Docker API
+    required: False
+    type: int
+    default: 180
 author: Jeffrey Zhang
 '''
 
@@ -78,11 +90,6 @@ EXAMPLES = '''
 '''
 
 
-import docker
-import json
-import re
-
-
 JSON_REG = re.compile('^(?P<host>\w+) \| (?P<status>\w+)!? =>(?P<stdout>.*)$',
                       re.MULTILINE | re.DOTALL)
 NON_JSON_REG = re.compile(('^(?P<host>\w+) \| (?P<status>\w+)!? \| '
@@ -114,14 +121,16 @@ def get_docker_client():
 
 def main():
     specs = dict(
-        module_name=dict(type='str'),
+        module_name=dict(required=True, type='str'),
         module_args=dict(type='str'),
         module_extra_vars=dict(type='json'),
-        api_version=dict(required=False, type='str', default='auto')
+        api_version=dict(required=False, type='str', default='auto'),
+        timeout=dict(required=False, type='int', default=180),
     )
     module = AnsibleModule(argument_spec=specs, bypass_checks=True)
     client = get_docker_client()(
-        version=module.params.get('api_version'))
+        version=module.params.get('api_version'),
+        timeout=module.params.get('timeout'))
     command_line = gen_commandline(module.params)
     kolla_toolbox = client.containers(filters=dict(name='kolla_toolbox',
                                                    status='running'))
@@ -136,6 +145,7 @@ def main():
         m = exp.match(output)
         if m:
             inner_output = m.groupdict().get('stdout')
+            status = m.groupdict().get('status')
             break
     else:
         module.fail_json(
@@ -145,11 +155,17 @@ def main():
     try:
         ret = json.loads(inner_output)
     except ValueError:
+        # Some modules (e.g. command) do not produce a JSON output. Instead,
+        # check the status, and assume changed on success.
         ret['stdout'] = inner_output
+        if status != "SUCCESS":
+            ret['failed'] = True
+        else:
+            # No way to know whether changed - assume yes.
+            ret['changed'] = True
 
     module.exit_json(**ret)
 
 
-from ansible.module_utils.basic import *  # noqa
 if __name__ == "__main__":
     main()
